@@ -10,6 +10,7 @@ use Drupal\Core\File\Exception\FileException;
 use Drupal\Core\File\FileSystemInterface;
 use Drupal\Core\Form\FormStateInterface;
 use Drupal\Core\StreamWrapper\StreamWrapperInterface;
+use Drupal\Core\StringTranslation\TranslatableMarkup;
 use Drupal\Core\TypedData\DataDefinition;
 use Drupal\file\Entity\File;
 use Drupal\file\Plugin\Field\FieldType\FileItem;
@@ -47,13 +48,6 @@ use Drupal\file\Plugin\Field\FieldType\FileItem;
  */
 class ImageItem extends FileItem {
 
-  /**
-   * The entity manager.
-   *
-   * @var \Drupal\Core\Entity\EntityManagerInterface
-   */
-  protected $entityManager;
-
   /**
    * {@inheritdoc}
    */
@@ -148,20 +142,20 @@ class ImageItem extends FileItem {
     unset($properties['description']);
 
     $properties['alt'] = DataDefinition::create('string')
-      ->setLabel(t('Alternative text'))
-      ->setDescription(t("Alternative image text, for the image's 'alt' attribute."));
+      ->setLabel(new TranslatableMarkup('Alternative text'))
+      ->setDescription(new TranslatableMarkup("Alternative image text, for the image's 'alt' attribute."));
 
     $properties['title'] = DataDefinition::create('string')
-      ->setLabel(t('Title'))
-      ->setDescription(t("Image title text, for the image's 'title' attribute."));
+      ->setLabel(new TranslatableMarkup('Title'))
+      ->setDescription(new TranslatableMarkup("Image title text, for the image's 'title' attribute."));
 
     $properties['width'] = DataDefinition::create('integer')
-      ->setLabel(t('Width'))
-      ->setDescription(t('The width of the image in pixels.'));
+      ->setLabel(new TranslatableMarkup('Width'))
+      ->setDescription(new TranslatableMarkup('The width of the image in pixels.'));
 
     $properties['height'] = DataDefinition::create('integer')
-      ->setLabel(t('Height'))
-      ->setDescription(t('The height of the image in pixels.'));
+      ->setLabel(new TranslatableMarkup('Height'))
+      ->setDescription(new TranslatableMarkup('The height of the image in pixels.'));
 
     return $properties;
   }
@@ -180,15 +174,15 @@ class ImageItem extends FileItem {
     $scheme_options = \Drupal::service('stream_wrapper_manager')->getNames(StreamWrapperInterface::WRITE_VISIBLE);
     $element['uri_scheme'] = [
       '#type' => 'radios',
-      '#title' => t('Upload destination'),
+      '#title' => $this->t('Upload destination'),
       '#options' => $scheme_options,
       '#default_value' => $settings['uri_scheme'],
-      '#description' => t('Select where the final files should be stored. Private file storage has significantly more overhead than public files, but allows restricted access to files within this field.'),
+      '#description' => $this->t('Select where the final files should be stored. Private file storage has significantly more overhead than public files, but allows restricted access to files within this field.'),
     ];
 
     // Add default_image element.
     static::defaultImageForm($element, $settings);
-    $element['default_image']['#description'] = t('If no image is uploaded, this image will be shown on display.');
+    $element['default_image']['#description'] = $this->t('If no image is uploaded, this image will be shown on display.');
 
     return $element;
   }
@@ -206,55 +200,55 @@ class ImageItem extends FileItem {
     $max_resolution = explode('x', $settings['max_resolution']) + ['', ''];
     $element['max_resolution'] = [
       '#type' => 'item',
-      '#title' => t('Maximum image resolution'),
-      '#element_validate' => [[get_class($this), 'validateResolution']],
+      '#title' => $this->t('Maximum image resolution'),
+      '#element_validate' => [[static::class, 'validateResolution']],
       '#weight' => 4.1,
-      '#field_prefix' => '<div class="container-inline">',
-      '#field_suffix' => '</div>',
-      '#description' => t('The maximum allowed image size expressed as WIDTH×HEIGHT (e.g. 640×480). Leave blank for no restriction. If a larger image is uploaded, it will be resized to reflect the given width and height. Resizing images on upload will cause the loss of <a href="http://wikipedia.org/wiki/Exchangeable_image_file_format">EXIF data</a> in the image.'),
+      '#description' => $this->t('The maximum allowed image size expressed as WIDTH×HEIGHT (e.g. 640×480). Leave blank for no restriction. If a larger image is uploaded, it will be resized to reflect the given width and height. Resizing images on upload will cause the loss of <a href="http://wikipedia.org/wiki/Exchangeable_image_file_format">EXIF data</a> in the image.'),
     ];
     $element['max_resolution']['x'] = [
       '#type' => 'number',
-      '#title' => t('Maximum width'),
+      '#title' => $this->t('Maximum width'),
       '#title_display' => 'invisible',
       '#default_value' => $max_resolution[0],
       '#min' => 1,
       '#field_suffix' => ' × ',
+      '#prefix' => '<div class="form--inline clearfix">',
     ];
     $element['max_resolution']['y'] = [
       '#type' => 'number',
-      '#title' => t('Maximum height'),
+      '#title' => $this->t('Maximum height'),
       '#title_display' => 'invisible',
       '#default_value' => $max_resolution[1],
       '#min' => 1,
-      '#field_suffix' => ' ' . t('pixels'),
+      '#field_suffix' => ' ' . $this->t('pixels'),
+      '#suffix' => '</div>',
     ];
 
     $min_resolution = explode('x', $settings['min_resolution']) + ['', ''];
     $element['min_resolution'] = [
       '#type' => 'item',
-      '#title' => t('Minimum image resolution'),
-      '#element_validate' => [[get_class($this), 'validateResolution']],
+      '#title' => $this->t('Minimum image resolution'),
+      '#element_validate' => [[static::class, 'validateResolution']],
       '#weight' => 4.2,
-      '#field_prefix' => '<div class="container-inline">',
-      '#field_suffix' => '</div>',
-      '#description' => t('The minimum allowed image size expressed as WIDTH×HEIGHT (e.g. 640×480). Leave blank for no restriction. If a smaller image is uploaded, it will be rejected.'),
+      '#description' => $this->t('The minimum allowed image size expressed as WIDTH×HEIGHT (e.g. 640×480). Leave blank for no restriction. If a smaller image is uploaded, it will be rejected.'),
     ];
     $element['min_resolution']['x'] = [
       '#type' => 'number',
-      '#title' => t('Minimum width'),
+      '#title' => $this->t('Minimum width'),
       '#title_display' => 'invisible',
       '#default_value' => $min_resolution[0],
       '#min' => 1,
       '#field_suffix' => ' × ',
+      '#prefix' => '<div class="form--inline clearfix">',
     ];
     $element['min_resolution']['y'] = [
       '#type' => 'number',
-      '#title' => t('Minimum height'),
+      '#title' => $this->t('Minimum height'),
       '#title_display' => 'invisible',
       '#default_value' => $min_resolution[1],
       '#min' => 1,
-      '#field_suffix' => ' ' . t('pixels'),
+      '#field_suffix' => ' ' . $this->t('pixels'),
+      '#suffix' => '</div>',
     ];
 
     // Remove the description option.
@@ -263,16 +257,16 @@ class ImageItem extends FileItem {
     // Add title and alt configuration options.
     $element['alt_field'] = [
       '#type' => 'checkbox',
-      '#title' => t('Enable <em>Alt</em> field'),
+      '#title' => $this->t('Enable <em>Alt</em> field'),
       '#default_value' => $settings['alt_field'],
-      '#description' => t('Short description of the image used by screen readers and displayed when the image is not loaded. Enabling this field is recommended.'),
+      '#description' => $this->t('Short description of the image used by screen readers and displayed when the image is not loaded. Enabling this field is recommended.'),
       '#weight' => 9,
     ];
     $element['alt_field_required'] = [
       '#type' => 'checkbox',
-      '#title' => t('<em>Alt</em> field required'),
+      '#title' => $this->t('<em>Alt</em> field required'),
       '#default_value' => $settings['alt_field_required'],
-      '#description' => t('Making this field required is recommended.'),
+      '#description' => $this->t('Making this field required is recommended.'),
       '#weight' => 10,
       '#states' => [
         'visible' => [
@@ -282,14 +276,14 @@ class ImageItem extends FileItem {
     ];
     $element['title_field'] = [
       '#type' => 'checkbox',
-      '#title' => t('Enable <em>Title</em> field'),
+      '#title' => $this->t('Enable <em>Title</em> field'),
       '#default_value' => $settings['title_field'],
-      '#description' => t('The title attribute is used as a tooltip when the mouse hovers over the image. Enabling this field is not recommended as it can cause problems with screen readers.'),
+      '#description' => $this->t('The title attribute is used as a tooltip when the mouse hovers over the image. Enabling this field is not recommended as it can cause problems with screen readers.'),
       '#weight' => 11,
     ];
     $element['title_field_required'] = [
       '#type' => 'checkbox',
-      '#title' => t('<em>Title</em> field required'),
+      '#title' => $this->t('<em>Title</em> field required'),
       '#default_value' => $settings['title_field_required'],
       '#weight' => 12,
       '#states' => [
@@ -301,7 +295,7 @@ class ImageItem extends FileItem {
 
     // Add default_image element.
     static::defaultImageForm($element, $settings);
-    $element['default_image']['#description'] = t("If no image is uploaded, this image will be shown on display and will override the field's default image.");
+    $element['default_image']['#description'] = $this->t("If no image is uploaded, this image will be shown on display and will override the field's default image.");
 
     return $element;
   }
@@ -358,12 +352,13 @@ class ImageItem extends FileItem {
         $image = File::create();
         $image->setFileUri($path);
         $image->setOwnerId(\Drupal::currentUser()->id());
-        $image->setMimeType(\Drupal::service('file.mime_type.guesser')->guess($path));
+        $guesser = \Drupal::service('file.mime_type.guesser');
+        $image->setMimeType($guesser->guessMimeType($path));
         $image->setFileName($file_system->basename($path));
         $destination_dir = static::doGetUploadLocation($settings);
         $file_system->prepareDirectory($destination_dir, FileSystemInterface::CREATE_DIRECTORY);
         $destination = $destination_dir . '/' . basename($path);
-        $file = file_move($image, $destination);
+        $file = \Drupal::service('file.repository')->move($image, $destination);
         $images[$extension][$min_resolution][$max_resolution][$file->id()] = $file;
       }
       else {
@@ -376,7 +371,7 @@ class ImageItem extends FileItem {
       $file = $images[$extension][$min_resolution][$max_resolution][$image_index];
     }
 
-    list($width, $height) = getimagesize($file->getFileUri());
+    [$width, $height] = getimagesize($file->getFileUri());
     $values = [
       'target_id' => $file->id(),
       'alt' => $random->sentences(4),
@@ -394,9 +389,9 @@ class ImageItem extends FileItem {
     if (!empty($element['x']['#value']) || !empty($element['y']['#value'])) {
       foreach (['x', 'y'] as $dimension) {
         if (!$element[$dimension]['#value']) {
-          // We expect the field name placeholder value to be wrapped in t()
+          // We expect the field name placeholder value to be wrapped in $this->t()
           // here, so it won't be escaped again as it's already marked safe.
-          $form_state->setError($element[$dimension], t('Both a height and width value must be specified in the @name field.', ['@name' => $element['#title']]));
+          $form_state->setError($element[$dimension], new TranslatableMarkup('Both a height and width value must be specified in the @name field.', ['@name' => $element['#title']]));
           return;
         }
       }
@@ -418,7 +413,7 @@ class ImageItem extends FileItem {
   protected function defaultImageForm(array &$element, array $settings) {
     $element['default_image'] = [
       '#type' => 'details',
-      '#title' => t('Default image'),
+      '#title' => $this->t('Default image'),
       '#open' => TRUE,
     ];
     // Convert the stored UUID to a FID.
@@ -429,27 +424,27 @@ class ImageItem extends FileItem {
     }
     $element['default_image']['uuid'] = [
       '#type' => 'managed_file',
-      '#title' => t('Image'),
-      '#description' => t('Image to be shown if no image is uploaded.'),
+      '#title' => $this->t('Image'),
+      '#description' => $this->t('Image to be shown if no image is uploaded.'),
       '#default_value' => $fids,
       '#upload_location' => $settings['uri_scheme'] . '://default_images/',
       '#element_validate' => [
         '\Drupal\file\Element\ManagedFile::validateManagedFile',
-        [get_class($this), 'validateDefaultImageForm'],
+        [static::class, 'validateDefaultImageForm'],
       ],
       '#upload_validators' => $this->getUploadValidators(),
     ];
     $element['default_image']['alt'] = [
       '#type' => 'textfield',
-      '#title' => t('Alternative text'),
-      '#description' => t('Short description of the image used by screen readers and displayed when the image is not loaded. This is important for accessibility.'),
+      '#title' => $this->t('Alternative text'),
+      '#description' => $this->t('Short description of the image used by screen readers and displayed when the image is not loaded. This is important for accessibility.'),
       '#default_value' => $settings['default_image']['alt'],
       '#maxlength' => 512,
     ];
     $element['default_image']['title'] = [
       '#type' => 'textfield',
-      '#title' => t('Title'),
-      '#description' => t('The title attribute is used as a tooltip when the mouse hovers over the image.'),
+      '#title' => $this->t('Title'),
+      '#description' => $this->t('The title attribute is used as a tooltip when the mouse hovers over the image.'),
       '#default_value' => $settings['default_image']['title'],
       '#maxlength' => 1024,
     ];
@@ -467,7 +462,7 @@ class ImageItem extends FileItem {
    * Validates the managed_file element for the default Image form.
    *
    * This function ensures the fid is a scalar value and not an array. It is
-   * assigned as a #element_validate callback in
+   * assigned as an #element_validate callback in
    * \Drupal\image\Plugin\Field\FieldType\ImageItem::defaultImageForm().
    *
    * @param array $element
@@ -481,7 +476,7 @@ class ImageItem extends FileItem {
     if (isset($element['fids']['#value'][0])) {
       $value = $element['fids']['#value'][0];
       // Convert the file ID to a uuid.
-      if ($file = \Drupal::entityManager()->getStorage('file')->load($value)) {
+      if ($file = \Drupal::entityTypeManager()->getStorage('file')->load($value)) {
         $value = $file->uuid();
       }
     }
@@ -499,16 +494,4 @@ class ImageItem extends FileItem {
     return TRUE;
   }
 
-  /**
-   * Gets the entity manager.
-   *
-   * @return \Drupal\Core\Entity\EntityManagerInterface
-   */
-  protected function getEntityManager() {
-    if (!isset($this->entityManager)) {
-      $this->entityManager = \Drupal::entityManager();
-    }
-    return $this->entityManager;
-  }
-
 }
