@@ -8,7 +8,9 @@ use Drupal\views\Views;
 use Drupal\views\Entity\View;
 
 /**
- * Relationship handler that allows a groupwise maximum of the linked in table.
+ * The relationship handler for groupwise maximum queries.
+ *
+ * It allows a groupwise maximum of the linked in table.
  * For a definition, see:
  * http://dev.mysql.com/doc/refman/5.0/en/example-maximum-column-group-row.html
  * In lay terms, instead of joining to get all matching records in the linked
@@ -59,6 +61,11 @@ use Drupal\views\Entity\View;
  */
 class GroupwiseMax extends RelationshipPluginBase {
 
+  /**
+   * The namespace of the subquery.
+   */
+  public string $subquery_namespace;
+
   /**
    * {@inheritdoc}
    */
@@ -196,7 +203,7 @@ class GroupwiseMax extends RelationshipPluginBase {
       // select field. See https://www.drupal.org/node/844910.
       // We work around this further down.
       $sort = $options['subquery_sort'];
-      list($sort_table, $sort_field) = explode('.', $sort);
+      [$sort_table, $sort_field] = explode('.', $sort);
       $sort_options = ['order' => $options['subquery_order']];
       $temp_view->addHandler('default', 'sort', $sort_table, $sort_field, $sort_options);
     }
@@ -215,7 +222,7 @@ class GroupwiseMax extends RelationshipPluginBase {
     $relationship_id = NULL;
     // Add the used relationship for the subjoin, if defined.
     if (isset($this->definition['relationship'])) {
-      list($relationship_table, $relationship_field) = explode(':', $this->definition['relationship']);
+      [$relationship_table, $relationship_field] = explode(':', $this->definition['relationship']);
       $relationship_id = $temp_view->addHandler('default', 'relationship', $relationship_table, $relationship_field);
     }
     $temp_item_options = ['relationship' => $relationship_id];
@@ -269,7 +276,7 @@ class GroupwiseMax extends RelationshipPluginBase {
     foreach ($orders as $order_key => $order) {
       // But if we're using a whole view, we don't know what we have!
       if ($options['subquery_view']) {
-        list($sort_table, $sort_field) = explode('.', $order_key);
+        [$sort_table, $sort_field] = explode('.', $order_key);
       }
       $orders[$sort_table . $this->subquery_namespace . '.' . $sort_field] = $order;
       unset($orders[$order_key]);
